#include "tSIOPortSkyeyeMain_tecsgen.h"
#include "target_serial.h"
#ifndef E_OK
#define	E_OK		0		/* 正常終了 */
#define	E_ID		(-18)	/* 不正ID番号 */
#endif


/* 属性アクセスマクロ */
/*
 * us_thr           void*            ATTR_us_thr     
 * us_rpr           void*            ATTR_us_rpr     
 * us_rcr           void*            ATTR_us_rcr     
 * us_csr           void*            ATTR_us_csr     
 * openflag         bool_t           VAR_openflag    
 * rcv_buf          char_t           VAR_rcv_buf     
 * rcv_erdy         bool_t           VAR_rcv_erdy    
 * snd_erdy         bool_t           VAR_snd_erdy    
 *
 * 呼び口関数 
 *
 * call port : ciSIOCBR  signature: siSIOCBR
 *   void           ciSIOCBR_readySend( );
 *   void           ciSIOCBR_readyReceive( );
 */
/* 受け口関数 
 *
 * entry port: eSIOPort
 * signature:  sSIOPort
 */

/*
 *  SIOドライバの初期化
 */
void
eInitialize_main(CELLIDX idx)
{
	CELLCB	*p_cellcb;

	assert(VALID_IDX(idx));
	p_cellcb = GET_CELLCB(idx);

	/*
	 *  シリアルI/Oポート管理ブロックの初期化
	 */
	VAR_openflag = false;
	/* 受信データの格納先アドレスの設定 */
	sil_wrw_mem((void *)(USART0_RPR), (uint32_t)(&VAR_rcv_buf));
    sil_wrw_mem((void *)(USART0_RCR), 0x0001);
}

/*
 *  SIOドライバの終了処理
 */
void
eTerminate_main(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	assert(VALID_IDX(idx));
	p_cellcb = GET_CELLCB(idx);

	/*
	 *  オープンされているシリアルI/Oポートのクローズ
	 */
	if (VAR_openflag) {
		/*
		 *  本当はeSIOPort_closeを呼びたい
		 */
		VAR_openflag = false;
		/*
		 *  シリアルI/O割込みをマスクする．
		 */
		dis_int(INTNO_SIO);
	}
}

/*
 * 文字を受信したか?
 */ 
Inline bool_t
uart_getready(CELLCB *p_cellcb)
{
	return(sil_rew_mem((void*)(ATTR_us_rcr)) == 0);
}
/*
 * 文字を送信できるか?
 */
Inline bool_t
uart_putready(CELLCB *p_cellcb)
{
    return(true);
}

/*
 * name:         eSIOPort_open
 * global_name:  tSIOPortSkyeyeMain_eSIOPort_open
 */
void
eSIOPort_open( CELLIDX idx)
{
	CELLCB	*p_cellcb;
	assert( VALID_IDX( idx ) );
	p_cellcb = GET_CELLCB(idx);
	
	VAR_openflag = true;
	ena_int(INTNO_SIO);
}

/*
 * name:         eSIOPort_close
 * global_name:  tSIOPortSkyeyeMain_eSIOPort_close
 */
void
eSIOPort_close( CELLIDX idx)
{
	CELLCB	*p_cellcb;
	assert( VALID_IDX( idx ) );
	p_cellcb = GET_CELLCB(idx);
	
	VAR_openflag = false;
	/*
	 *  シリアルI/O割込みをマスクする．
	 */
	dis_int(INTNO_SIO);
}

/*
 * 送信する文字の書き込み 
 * name:         eSIOPort_putchar
 * global_name:  tSIOPortSkyeyeMain_eSIOPort_putchar
 */
bool_t
eSIOPort_putChar( CELLIDX idx, char_t c)
{
	CELLCB	*p_cellcb;
	assert( VALID_IDX( idx ) );
	p_cellcb = GET_CELLCB(idx);

	if (uart_putready(p_cellcb)){
		sil_wrw_mem((void*)(ATTR_us_thr),c); 
        return(true);
    }
    return(false);
}

/*
 * 受信した文字の取り出し
 * name:         eSIOPort_getchar
 * global_name:  tSIOPortSkyeyeMain_eSIOPort_getchar
 */
int_t
eSIOPort_getChar(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	char_t c;
	
	assert( VALID_IDX( idx ) );
	p_cellcb = GET_CELLCB(idx);
	if (uart_getready(p_cellcb)) {
		c = VAR_rcv_buf;
		sil_wrw_mem((void *)(ATTR_us_rpr), (uint32_t)(&VAR_rcv_buf));
		sil_wrw_mem((void *)(ATTR_us_rcr), 0x0001);
		return (c);
	}
	return (-1);
}

/*
 * シリアルI/Oポートからのコールバックの許可
 * name:         eSIOPort_enableCBR
 * global_name:  tSIOPortSkyeyeMain_eSIOPort_enableCBR
 */
void
eSIOPort_enableCBR( CELLIDX idx, uint_t cbrtn)
{
	CELLCB	*p_cellcb;

	assert( VALID_IDX( idx ) );
	p_cellcb = GET_CELLCB(idx);

	switch (cbrtn) {
	  case SIO_RDY_SND:
		VAR_snd_ready = true;
		break;
	  case SIO_RDY_RCV:
		VAR_rcv_ready = true;
		break;
	}

}

/*
 * シリアルI/Oポートからのコールバックの禁止 
 * name:         eSIOPort_disableCBR
 * global_name:  tSIOPortSkyeyeMain_eSIOPort_disableCBR
 */
void
eSIOPort_disableCBR( CELLIDX idx, uint_t cbrtn)
{
	CELLCB	*p_cellcb;

	assert( VALID_IDX( idx ) );
	p_cellcb = GET_CELLCB(idx);
	switch (cbrtn) {
	  case SIO_RDY_SND:
		VAR_snd_ready = false;
		break;
	  case SIO_RDY_RCV:
		VAR_rcv_ready = false;
		break;
	}
}


/*
 *  SIOの割込みハンドラ
 */
void
eiISR_main(CELLIDX idx)
{
	CELLCB	*p_cellcb;

	assert( VALID_IDX( idx ) );
	p_cellcb = GET_CELLCB(idx);

	if(VAR_rcv_ready){
		/*
		 *  受信通知コールバックルーチンを呼び出す．
		 */
		ciSIOCBR_readyReceive();
	}
	if(VAR_snd_ready){
		/*
		 *  送信可能コールバックルーチンを呼び出す．
		 */
		ciSIOCBR_readySend();
	}

}
