#include "tShmemHalfChannel_tecsgen.h"

static ER       tShmemHalfChan_send_wait( TSHMEMHALFCHAN_CH ch, tShmemHalfChannel_CB *p_cellcb, FLGPTN snd_flgptn, TMO tmo );
static ER       tShmemHalfChan_recv_wait( TSHMEMHALFCHAN_CH ch, tShmemHalfChannel_CB *p_cellcb, FLGPTN rcv_flgptn, TMO tmo );
static ER       tShmemHalfChan_write_buf( tShmemHalfChannel_CB *p_cellcb, const int_least8_t *buf, int16_t size );
static ER_UINT  tShmemHalfChan_read_buf(  tShmemHalfChannel_CB *p_cellcb, int_least8_t *buf, int16_t size );
static void     tShmemHalfChan_end_receive( tShmemHalfChannel_CB *p_cellcb );

/* °ޥ */
/*
 * size             int32_t          ATTR_size       
 * buf              int_least8_t*    VAR_buf         
 * count            int32_t          VAR_count       
 * rp               int32_t          VAR_rp          
 * b_reset1         tecs_bool        VAR_b_reset1    
 * b_reset2         tecs_bool        VAR_b_reset2    
 */

/* ƤӸؿ */
/*
 * call port : cSem  signature: sSem
 *   ER             cSem_sig_sem( );
 *   ER             cSem_isig_sem( );
 *   ER             cSem_wai_sem( );
 *   ER             cSem_pol_sem( );
 *   ER             cSem_twai_sem( TMO tmout );
 * call port : cFlg  signature: sFlg
 *   ER             cFlg_set_flg( subscript, FLGPTN setptn );
 *   ER             cFlg_clr_flg( subscript, FLGPTN clrptn );
 *   ER             cFlg_wai_flg( subscript, FLGPTN waiptn, MODE wfmode, FLGPTN* p_flgptn );
 *   ER             cFlg_pol_flg( subscript, FLGPTN waiptn, MODE wfmode, FLGPTN* p_flgptn );
 *   ER             cFlg_twai_flg( subscript, FLGPTN waiptn, MODE wfmode, FLGPTN* p_flgptn, TMO tmout );
 */

/* ؿ */
/*
 * entry port: eC0
 * signature:  sChannel
 */
/*
 * name:         eC0_open
 * global_name:  tShmemHalfChannel_eC0_open
 */
ER             eC0_open( tShmemHalfChannel_IDX idx, const int_least8_t* arg, int16_t size, TMO tmo)
{
	struct tag_tShmemHalfChannel_CB *p_cellcb;
	if( tShmemHalfChannel_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return E_ID;
	}
  	(void)cSem_wai_sem();
	(void)cFlg_clr_flg( 0, 0x0 );
	(void)cFlg_set_flg( 0, TSHMEMHALFCHAN_FLG_WAIT );
	(void)cFlg_clr_flg( 1, 0x0 );
	(void)cFlg_set_flg( 1, TSHMEMHALFCHAN_FLG_WAIT );
	(void)cSem_sig_sem();

	return E_OK;
}

/*
 * name:         eC0_close
 * global_name:  tShmemHalfChannel_eC0_close
 */
ER             eC0_close( tShmemHalfChannel_IDX idx)
{
  struct tag_tShmemHalfChannel_CB *p_cellcb;
  if( tShmemHalfChannel_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
     return E_ID;
  }
  return E_OK;
}

/*
 * name:         eC0_flush
 * global_name:  tShmemHalfChannel_eC0_flush
 */
ER             eC0_flush( tShmemHalfChannel_IDX idx)
{
  struct tag_tShmemHalfChannel_CB *p_cellcb;
  if( tShmemHalfChannel_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
     return E_ID;
  }
  if( VAR_b_reset2 == tecs_true ){
      VAR_b_reset2 = tecs_false;
	  return  E_RESET;
  }

  cFlg_clr_flg( 0, 0x0 );
  cFlg_set_flg( 0, TSHMEMHALFCHAN_FLG_RCV1 );
  cFlg_clr_flg( 1, 0x0 );
  cFlg_set_flg( 1, TSHMEMHALFCHAN_FLG_RCV1 );
  return E_OK;
}

/*
 * name:         eC0_reset
 * global_name:  tShmemHalfChannel_eC0_reset
 */
ER             eC0_reset( tShmemHalfChannel_IDX idx)
{
  struct tag_tShmemHalfChannel_CB *p_cellcb;
  if( tShmemHalfChannel_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
     return E_ID;
  }
  VAR_b_reset2 = tecs_false;
  VAR_b_reset1 = tecs_true;
  return E_OK;
}

/*
 * name:         eC0_send
 * global_name:  tShmemHalfChannel_eC0_send
 */
ER             eC0_send( tShmemHalfChannel_IDX idx, const int_least8_t* buf, int16_t size, TMO tmo)
{
    struct tag_tShmemHalfChannel_CB *p_cellcb;
	ER    ercd;

	if( tShmemHalfChannel_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return E_ID;
	}

	/* channel 1 񤭤ǤΤԤ */
    if( ( ercd = tShmemHalfChan_send_wait( 0, p_cellcb, TSHMEMHALFCHAN_FLG_SND0, tmo ) ) != E_OK )
		return  ercd;

	/* Хåե˽񤭤 */
    tShmemHalfChan_write_buf( p_cellcb, buf, size );

	return ercd;
}

/*
 * name:         eC0_receive
 * global_name:  tShmemHalfChannel_eC0_receive
 */
ER_UINT          eC0_receive( tShmemHalfChannel_IDX idx, int_least8_t* buf, int16_t size, TMO tmo)
{
	struct tag_tShmemHalfChannel_CB *p_cellcb;
	FLGPTN   flgptn;
	ER_UINT  er_sz;

	if( tShmemHalfChannel_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return E_ID;
	}
	if( VAR_b_reset2 == tecs_true ){
		VAR_b_reset2 = tecs_false;
		return  E_RESET;
	}

	/* Ԥ */
    if( ( er_sz = tShmemHalfChan_recv_wait( 0, p_cellcb, TSHMEMHALFCHAN_FLG_RCV0, tmo ) ) != E_OK )
		return  er_sz;

	/* Хåեɤ߽Ф */
    er_sz = tShmemHalfChan_read_buf( p_cellcb, buf, size );

	return er_sz;
}

/*
 * name:         eC0_end_receive
 * global_name:  tShmemHalfChannel_eC0_end_receive
 */
ER             eC0_end_receive( tShmemHalfChannel_IDX idx)
{
    ER       ercd_ = E_OK;
    tShmemHalfChannel_CB *p_cellcb;
    if( tShmemHalfChannel_VALID_IDX( idx ) ){
        p_cellcb = GET_CELLCB(idx);
    }else{
        return E_ID;
    }

	tShmemHalfChan_end_receive( p_cellcb );
    return ercd_;
}

/*
 * entry port: eC1
 * signature:  sChannel
 */
/*
 * name:         eC1_open
 * global_name:  tShmemHalfChannel_eC1_open
 */
ER             eC1_open( tShmemHalfChannel_IDX idx, const int_least8_t* arg, int16_t size, TMO tmo)
{
  struct tag_tShmemHalfChannel_CB *p_cellcb;
  if( tShmemHalfChannel_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
     return E_ID;
  }
  return E_OK;
}

/*
 * name:         eC1_close
 * global_name:  tShmemHalfChannel_eC1_close
 */
ER             eC1_close( tShmemHalfChannel_IDX idx)
{
  struct tag_tShmemHalfChannel_CB *p_cellcb;
  if( tShmemHalfChannel_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
     return E_ID;
  }
  return E_OK;
}

/*
 * name:         eC1_flush
 * global_name:  tShmemHalfChannel_eC1_flush
 */
ER             eC1_flush( tShmemHalfChannel_IDX idx)
{
	struct tag_tShmemHalfChannel_CB *p_cellcb;
	if( tShmemHalfChannel_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return E_ID;
	}

	cFlg_clr_flg( 0, 0x0 );
	cFlg_set_flg( 0, TSHMEMHALFCHAN_FLG_RCV0 );
	cFlg_clr_flg( 1, 0x0 );
	cFlg_set_flg( 1, TSHMEMHALFCHAN_FLG_RCV0 );
	return E_OK;
}

/*
 * name:         eC1_reset
 * global_name:  tShmemHalfChannel_eC1_reset
 */
ER             eC1_reset( tShmemHalfChannel_IDX idx)
{
  struct tag_tShmemHalfChannel_CB *p_cellcb;
  if( tShmemHalfChannel_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
     return E_ID;
  }
  VAR_b_reset2 = tecs_true;
  return E_OK;
}

/*
 * name:         eC1_send
 * global_name:  tShmemHalfChannel_eC1_send
 */
ER             eC1_send( tShmemHalfChannel_IDX idx, const int_least8_t* buf, int16_t size, TMO tmo)
{
	ER     ercd;
	struct tag_tShmemHalfChannel_CB *p_cellcb;
	if( tShmemHalfChannel_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return E_ID;
	}

	/* channel 1 񤭤ǤΤԤ */
    if( ( ercd = tShmemHalfChan_send_wait( 1, p_cellcb, TSHMEMHALFCHAN_FLG_SND1, tmo ) ) != E_OK )
		return  ercd;

	/* Хåե˽񤭤 */
    tShmemHalfChan_write_buf( p_cellcb, buf, size );

	return ercd;
}

/*
 * name:         eC1_receive
 * global_name:  tShmemHalfChannel_eC1_receive
 */
ER_UINT          eC1_receive( tShmemHalfChannel_IDX idx, int_least8_t* buf, int16_t size, TMO tmo)
{
	ER     er_sz;
	struct tag_tShmemHalfChannel_CB *p_cellcb;
	if( tShmemHalfChannel_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return E_ID;
	}

	if( VAR_b_reset2 == tecs_true ){
		VAR_b_reset2 = tecs_false;
		return  E_RESET;
	}

	/* Ԥ */
    if( ( er_sz = tShmemHalfChan_recv_wait( 1, p_cellcb, TSHMEMHALFCHAN_FLG_RCV1, tmo ) ) != E_OK )
		return  er_sz;

	/* Хåեɤ߽Ф */
    er_sz = tShmemHalfChan_read_buf( p_cellcb, buf, size );

	return er_sz;
}


/*
 * name:         eC1_end_receive
 * global_name:  tShmemHalfChannel_eC1_end_receive
 */
ER             eC1_end_receive( tShmemHalfChannel_IDX idx)
{
    ER       ercd_ = E_OK;
    tShmemHalfChannel_CB *p_cellcb;
    if( tShmemHalfChannel_VALID_IDX( idx ) ){
        p_cellcb = GET_CELLCB(idx);
    }else{
        return E_ID;
    }

	tShmemHalfChan_end_receive( p_cellcb );
    return ercd_;
}


/*
 * name:  tShmemHalfChan_send_wait
 * param: IN FLGPTN snd_flgptn   TSHMEMHALFCHAN_FLG_SND0 or TSHMEMHALFCHAN_FLG_SND1
 * param: IN TMO    tmo          ॢȻ
 * REM:
 *  Ԥ
 *     cFlg  snd_flgptn ξ֤ΤȤľ E_OK ֤
 *    cFlg  TSHMEMHALFCHAN_FLG_WAIT ΤȤcFlg  snd_flgptn ѹ (κ cSem ¾椹)
 *  tmo ǸƤӽФġΥƥॳϤ뤿ᡢǰξ tmo Ķ뤳Ȥꤦ
 *  ʤΤ褦ʤȤΤϡޤ졥 mikan ¬ꤷॢȤˤ٤
 */
static ER  tShmemHalfChan_send_wait( TSHMEMHALFCHAN_CH ch, tShmemHalfChannel_CB *p_cellcb, FLGPTN snd_flgptn, TMO tmo )
{
	FLGPTN     flgptn;
	ER         ercd;
	tecs_bool  b_ready = tecs_false;

	/* ǤƤ뤫 */
	if( ( ercd = cFlg_twai_flg( ch, snd_flgptn, TWF_ANDW, &flgptn, TMO_POL ) ) != E_TMOUT ){
		/* E_OK ޤϡ顼(return ) */

	}else{
		/* ޤ֤ˤʤäƤʤ */

		do {
			/* ̤Ѿ֤ˤʤΤԤġ */
			if( ( ercd = cFlg_twai_flg( ch, TSHMEMHALFCHAN_FLG_WAIT, TWF_ANDW, &flgptn, tmo ) ) == E_OK){

				/* ƱԤ */
				if( ( ercd = cSem_twai_sem( tmo ) ) != E_OK ){
					/* ॢȤ顼Ǥڤ */
					break;
				}

				/* ̤Ѿ֤ ʥޥեޤǤ˥åȤƤʤå */
				if( ( ercd = cFlg_twai_flg( ch, TSHMEMHALFCHAN_FLG_WAIT, TWF_ANDW, &flgptn, TMO_POL ) ) == E_OK ){
					/* ե饰򥻥å */
					(void)cFlg_set_flg( 0, snd_flgptn );
					(void)cFlg_set_flg( 1, snd_flgptn );
					b_ready = tecs_true;
				}

				/* */  
				(void)cSem_sig_sem();
			} else {
				/* 顼ʴޤ E_TMOUT) */
				break;
			}
		}while( b_ready == tecs_false );
	}
	return ercd;
}

/*
 * name:  tShmemHalfChan_recv_wait
 * param: IN FLGPTN rcv_flgptn   TSHMEMHALFCHAN_FLG_RCV0 or TSHMEMHALFCHAN_FLG_RCV1
 * param: IN TMO    tmo          ॢȻ
 * REM:
 *  Ԥ
 *  ̾ tmo ǸƤӽФġΥƥॳϤ뤿ᡢǰξ tmo Ķ뤳Ȥꤦ
 *  ʤΤ褦ʤȤΤϡޤ
 */
static ER  tShmemHalfChan_recv_wait( TSHMEMHALFCHAN_CH ch, tShmemHalfChannel_CB *p_cellcb, FLGPTN rcv_flgptn, TMO tmo )
{
	FLGPTN     flgptn;
	ER         ercd;

	/* ǤƤ뤫 */
	ercd = cFlg_twai_flg( ch, rcv_flgptn, TWF_ANDW, &flgptn, tmo );
		/* E_OK ޤϡ顼(return ) */
	return ercd;
}


static ER    tShmemHalfChan_write_buf( tShmemHalfChannel_CB *p_cellcb, const int_least8_t *buf, int16_t size )
{
	ER   ercd;

	/* ХåեĶ񤭹ߤ ? */
	if( VAR_rp + size >= ATTR_size )
		ercd = E_BOVR;
	else {
		memcpy( &VAR_buf[ VAR_count ], buf, size );
		VAR_count += size;
		ercd = E_OK;
	}
	return ercd;
}


static ER_UINT   tShmemHalfChan_read_buf( tShmemHalfChannel_CB *p_cellcb, int_least8_t *buf, int16_t size )
{
	/* ɽФĹη */
	if( VAR_rp + size > VAR_count )
		size = VAR_count - VAR_rp;

	memcpy( buf, &VAR_buf[ VAR_rp ], size );
	VAR_rp += size;

	return (ER_UINT)size;
}

static  void     tShmemHalfChan_end_receive( tShmemHalfChannel_CB *p_cellcb )
{
	(void)cSem_wai_sem();
	(void)cFlg_clr_flg( 0, 0x0 );
	(void)cFlg_set_flg( 0, TSHMEMHALFCHAN_FLG_WAIT );
	(void)cFlg_clr_flg( 1, 0x0 );
	(void)cFlg_set_flg( 1, TSHMEMHALFCHAN_FLG_WAIT );
	(void)cSem_sig_sem();
}

